/* testlist 

  Copyright (C) 1998, 2002, 2003 Martin A. Olevitch
  Written by Martin A. Olevitch (marty@cosray.wustl.edu)

  This file is part of Linklist.

  Linklist is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Linklist is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Linklist, see the file COPYING; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "linklist.h"

struct datum {
    char first[20];
    char last[20];
    int id;
};

#define MAX	100

int main(int argc, char **argv)
{
    int cnt;
    struct datum *d;
    struct datum **dpp;
    int i;
    int id;
    char line[MAX];
    int n;
    int ret;

    cnt = 0;
    printf("> ");
    while (gets(line) != NULL) {
	switch(line[0]) {
	    case '?':
	    	printf("0 init    n new    D destroy    f first\n");
		printf("l last    + next   - prev       v val\n");
		printf("d delete  p print  I id         M multi\n");
		printf("b insert before    a insert after\n");
		printf("q quit\n");
		break;
	    case 'q':
	    	exit(0);
		break;
	    case 'I':
	    	printf("id? ");
		gets(line);
		id = atoi(line);
		break;
	    case '0':
	    	printf("init\n");
		printf("number of lists? ");
		gets(line);
		n = atoi(line);
	    	ret = ll_init(n);
		if (ret != 0) {
		    printf("bad ll_init (%d)\n", ret);
		}
		break;
	    case 'n':
	    	printf("new\n");
		id = ll_new_list();
		if (id < 0) {
		    printf("bad ll_new_list (%d)\n", id);
		} else {
		    printf("ID %d\n", id);
		}
		break;
	    case 'D':
	    	printf("destroy\n");
		dpp = malloc(cnt * sizeof(struct datum *));
		if (dpp == NULL) {
		    printf("can't malloc\n");
		    break;
		}
		ret = ll_destroy_list(id, (void **)dpp);
		printf("ret = %d\n", ret);
		if (ret < 0) {
		    break;
		}
		for (i=0; i<ret; i++) {
		    free(dpp[i]);
		}
		free(dpp);
		break;
	    case 'M':
	    	printf("multiple insert\n");
		printf("How many inserts? ");
		gets(line);
		n = atoi(line);
		d = malloc(sizeof (struct datum));
		if (d == NULL) {
		    printf("can't malloc\n");
		    break;
		}
		printf("first name? ");
		gets(line);
		strcpy(d->first, line);
		printf("last name? ");
		gets(line);
		strcpy(d->last, line);
		printf("ID number? ");
		gets(line);
		d->id = atoi(line);
		for (i=0; i<n; i++) {
		    ret = ll_insert(id, d);
		    if (ret) {
			printf("ll_insert: %d (rep %d)\n", ret, cnt);
		    }
		    cnt++;
		}
		break;
	    case 'a':
	    	printf("insert after\n");
		d = malloc(sizeof (struct datum));
		if (d == NULL) {
		    printf("can't malloc\n");
		    break;
		}
		printf("first name? ");
		gets(line);
		strcpy(d->first, line);
		printf("last name? ");
		gets(line);
		strcpy(d->last, line);
		printf("ID number? ");
		gets(line);
		d->id = atoi(line);
		ret = ll_insert_after(id, d);
		if (ret) {
		    printf("ll_insert_after returned %d\n", ret);
		}
		cnt++;
		break;
	    case 'b':
	    	printf("insert before\n");
		d = malloc(sizeof (struct datum));
		if (d == NULL) {
		    printf("can't malloc\n");
		    break;
		}
		printf("first name? ");
		gets(line);
		strcpy(d->first, line);
		printf("last name? ");
		gets(line);
		strcpy(d->last, line);
		printf("ID number? ");
		gets(line);
		d->id = atoi(line);
		ret = ll_insert_before(id, d);
		if (ret) {
		    printf("ll_insert_before returned %d\n", ret);
		}
		cnt++;
		break;
	    case 'd':
	    	printf("delete\n");
	    	d = ll_delete(id);
		if (d == NULL) {
		    printf("NULL\n");
		} else {
		    printf("first: %s\n", d->first);
		    printf("last: %s\n", d->last);
		    printf("id: %d\n", d->id);
		    free(d);
		    cnt--;
		}
		break;
	    case 'f':
	    	printf("first\n");
	    	ret = ll_first(id);
		printf("ret %d\n", ret);
		break;
	    case 'l':
	    	printf("last\n");
	    	ret = ll_last(id);
		printf("ret %d\n", ret);
		break;
	    case '+':
	    	printf("next\n");
	    	ret = ll_next(id);
		printf("ret %d\n", ret);
		break;
	    case '-':
	    	printf("prev\n");
	    	ret = ll_prev(id);
		printf("ret %d\n", ret);
		break;
	    case 'v':
	    	printf("val\n");
	    	d = ll_val(id);
		if (d == NULL) {
		    printf("NULL\n");
		} else {
		    printf("first: %s\n", d->first);
		    printf("last: %s\n", d->last);
		    printf("id: %d\n", d->id);
		}
		break;
	    case 'p':
	    	printf("print\n");
	    	printf("not yet implemented\n");
		break;
	    default:
	    	printf("unknown cmd '%c'\n", line[0]);
		break;
	}
	printf("> ");
    }
}

